@@ -1,11 +1,8 @@
 require "helper"
 
-class Application; end
-module ApplicationHelper; end
-
 describe Thor::Actions do
-  def runner(options = {})
-    @runner ||= MyCounter.new([1], options, :destination_root => destination_root)
+  def runner(options = {}, behavior = :invoke)
+    @runner ||= MyCounter.new([1], options, :destination_root => destination_root, :behavior => behavior)
   end
 
   def action(*args, &block)
@@ -259,6 +256,13 @@ describe Thor::Actions do
         expect(File.exist?(file)).to be false
       end
 
+      it "removes broken symlinks too" do
+        link_path = File.join(destination_root, "broken_symlink")
+        ::FileUtils.ln_s("invalid_reference", link_path)
+        action :remove_file, "broken_symlink"
+        expect(File.symlink?(link_path) || File.exist?(link_path)).to be false
+      end
+
       it "removes directories too" do
         action :remove_dir, "doc"
         expect(File.exist?(File.join(destination_root, "doc"))).to be false
@@ -280,28 +284,92 @@ describe Thor::Actions do
     end
 
     describe "#gsub_file" do
-      it "replaces the content in the file" do
-        action :gsub_file, "doc/README", "__start__", "START"
-        expect(File.binread(file)).to eq("START\nREADME\n__end__\n")
+      context "with invoke behavior" do
+        it "replaces the content in the file" do
+          action :gsub_file, "doc/README", "__start__", "START"
+          expect(File.binread(file)).to eq("START\nREADME\n__end__\n")
+        end
+
+        it "does not replace if pretending" do
+          runner(:pretend => true)
+          action :gsub_file, "doc/README", "__start__", "START"
+          expect(File.binread(file)).to eq("__start__\nREADME\n__end__\n")
+        end
+
+        it "accepts a block" do
+          action(:gsub_file, "doc/README", "__start__") { |match| match.gsub("__", "").upcase }
+          expect(File.binread(file)).to eq("START\nREADME\n__end__\n")
+        end
+
+        it "logs status" do
+          expect(action(:gsub_file, "doc/README", "__start__", "START")).to eq("        gsub  doc/README\n")
+        end
+
+        it "does not log status if required" do
+          expect(action(:gsub_file, file, "__", :verbose => false) { |match| match * 2 }).to be_empty
+        end
       end
 
-      it "does not replace if pretending" do
-        runner(:pretend => true)
-        action :gsub_file, "doc/README", "__start__", "START"
-        expect(File.binread(file)).to eq("__start__\nREADME\n__end__\n")
-      end
+      context "with revoke behavior" do
+        context "and no force option" do
+          it "does not replace the content in the file" do
+            runner({}, :revoke)
+            action :gsub_file, "doc/README", "__start__", "START"
+            expect(File.binread(file)).to eq("__start__\nREADME\n__end__\n")
+          end
 
-      it "accepts a block" do
-        action(:gsub_file, "doc/README", "__start__") { |match| match.gsub("__", "").upcase }
-        expect(File.binread(file)).to eq("START\nREADME\n__end__\n")
-      end
+          it "does not replace if pretending" do
+            runner({ :pretend => true }, :revoke)
+            action :gsub_file, "doc/README", "__start__", "START"
+            expect(File.binread(file)).to eq("__start__\nREADME\n__end__\n")
+          end
 
-      it "logs status" do
-        expect(action(:gsub_file, "doc/README", "__start__", "START")).to eq("        gsub  doc/README\n")
-      end
+          it "does not replace the content in the file when given a block" do
+            runner({}, :revoke)
+            action(:gsub_file, "doc/README", "__start__") { |match| match.gsub("__", "").upcase }
+            expect(File.binread(file)).to eq("__start__\nREADME\n__end__\n")
+          end
 
-      it "does not log status if required" do
-        expect(action(:gsub_file, file, "__", :verbose => false) { |match| match * 2 }).to be_empty
+          it "does not log status" do
+            runner({}, :revoke)
+            expect(action(:gsub_file, "doc/README", "__start__", "START")).to be_empty
+          end
+
+          it "does not log status if required" do
+            runner({}, :revoke)
+            expect(action(:gsub_file, file, "__", :verbose => false) { |match| match * 2 }).to be_empty
+          end
+        end
+
+        context "and force option" do
+          it "replaces the content in the file" do
+            runner({}, :revoke)
+            action :gsub_file, "doc/README", "__start__", "START", :force => true
+            expect(File.binread(file)).to eq("START\nREADME\n__end__\n")
+          end
+
+          it "does not replace if pretending" do
+            runner({ :pretend => true }, :revoke)
+            action :gsub_file, "doc/README", "__start__", "START", :force => true
+            expect(File.binread(file)).to eq("__start__\nREADME\n__end__\n")
+          end
+
+          it "replaces the content in the file when given a block" do
+            runner({}, :revoke)
+            action(:gsub_file, "doc/README", "__start__", :force => true) { |match| match.gsub("__", "").upcase }
+            expect(File.binread(file)).to eq("START\nREADME\n__end__\n")
+          end
+
+          it "logs status" do
+            runner({}, :revoke)
+            expect(action(:gsub_file, "doc/README", "__start__", "START", :force => true)).to eq("        gsub  doc/README\n")
+          end
+
+          it "does not log status if required" do
+            runner({}, :revoke)
+            expect(action(:gsub_file, file, "__", :verbose => false, :force => true) { |match| match * 2 }).to be_empty
+          end
+        end
       end
     end
 
@@ -343,17 +411,17 @@ describe Thor::Actions do
       end
 
       it "appends content to a class" do
-        action :inject_into_class, "application.rb", Application, "  filter_parameters :password\n"
+        action :inject_into_class, "application.rb", "Application", "  filter_parameters :password\n"
         expect(File.binread(file)).to eq("class Application < Base\n  filter_parameters :password\nend\n")
       end
 
       it "accepts a block" do
-        action(:inject_into_class, "application.rb", Application) { "  filter_parameters :password\n" }
+        action(:inject_into_class, "application.rb", "Application") { "  filter_parameters :password\n" }
         expect(File.binread(file)).to eq("class Application < Base\n  filter_parameters :password\nend\n")
       end
 
       it "logs status" do
-        expect(action(:inject_into_class, "application.rb", Application, "  filter_parameters :password\n")).to eq("      insert  application.rb\n")
+        expect(action(:inject_into_class, "application.rb", "Application", "  filter_parameters :password\n")).to eq("      insert  application.rb\n")
       end
 
       it "does not append if class name does not match" do
@@ -368,17 +436,17 @@ describe Thor::Actions do
       end
 
       it "appends content to a module" do
-        action :inject_into_module, "application_helper.rb", ApplicationHelper, "  def help; 'help'; end\n"
+        action :inject_into_module, "application_helper.rb", "ApplicationHelper", "  def help; 'help'; end\n"
         expect(File.binread(file)).to eq("module ApplicationHelper\n  def help; 'help'; end\nend\n")
       end
 
       it "accepts a block" do
-        action(:inject_into_module, "application_helper.rb", ApplicationHelper) { "  def help; 'help'; end\n" }
+        action(:inject_into_module, "application_helper.rb", "ApplicationHelper") { "  def help; 'help'; end\n" }
         expect(File.binread(file)).to eq("module ApplicationHelper\n  def help; 'help'; end\nend\n")
       end
 
       it "logs status" do
-        expect(action(:inject_into_module, "application_helper.rb", ApplicationHelper, "  def help; 'help'; end\n")).to eq("      insert  application_helper.rb\n")
+        expect(action(:inject_into_module, "application_helper.rb", "ApplicationHelper", "  def help; 'help'; end\n")).to eq("      insert  application_helper.rb\n")
       end
 
       it "does not append if module name does not match" do
