@@ -165,11 +165,19 @@ describe Thor::Actions do
       end
     end
 
+    it "returns the value yielded by the block" do
+      expect(runner.inside("foo") { 123 }).to eq(123)
+    end
+
     describe "when pretending" do
       it "no directories should be created" do
         runner.inside("bar", :pretend => true) {}
         expect(File.exist?("bar")).to be false
       end
+
+      it "returns the value yielded by the block" do
+        expect(runner.inside("foo") { 123 }).to eq(123)
+      end
     end
 
     describe "when verbose" do
@@ -231,13 +239,17 @@ describe Thor::Actions do
 
     it "accepts a URL as the path" do
       @file = "http://gist.github.com/103208.txt"
-      expect(runner).to receive(:open).with(@file, "Accept" => "application/x-thor-template").and_return(@template)
+      stub_request(:get, @file)
+
+      expect(runner).to receive(:apply).with(@file).and_return(@template)
       action(:apply, @file)
     end
 
     it "accepts a secure URL as the path" do
       @file = "https://gist.github.com/103208.txt"
-      expect(runner).to receive(:open).with(@file, "Accept" => "application/x-thor-template").and_return(@template)
+      stub_request(:get, @file)
+
+      expect(runner).to receive(:apply).with(@file).and_return(@template)
       action(:apply, @file)
     end
 
