@@ -263,6 +263,8 @@ describe Thor::Options do
         expect(parse("-f=12")["foo"]).to eq("12")
         expect(parse("--foo=12")["foo"]).to eq("12")
         expect(parse("--foo=bar=baz")["foo"]).to eq("bar=baz")
+        expect(parse("--foo=-bar")["foo"]).to eq("-bar")
+        expect(parse("--foo=-bar -baz")["foo"]).to eq("-bar -baz")
       end
 
       it "must accept underscores switch=value assignment" do
@@ -302,11 +304,10 @@ describe Thor::Options do
             "Expected '--fruit' to be one of #{enum.join(', ')}; got orange")
       end
 
-      it "allows multiple values if repeatable is specified" do
-        create :foo => Thor::Option.new("foo", :type => :string, :repeatable => true)
-
+      it "does not erroneously mutate defaults" do
+        create :foo => Thor::Option.new("foo", :type => :string, :repeatable => true, :required => false, :default => [])
         expect(parse("--foo=bar", "--foo", "12")["foo"]).to eq(["bar", "12"])
-        expect(parse("--foo", "13", "--foo", "14")["foo"]).to eq(["bar", "12", "13", "14"])
+        expect(@opt.instance_variable_get(:@switches)["--foo"].default).to eq([])
       end
     end
 
@@ -399,6 +400,7 @@ describe Thor::Options do
 
       it "accepts a switch=<value> assignment" do
         expect(parse("--attributes=name:string", "age:integer")["attributes"]).to eq("name" => "string", "age" => "integer")
+        expect(parse("--attributes=-name:string", "age:integer", "--gender:string")["attributes"]).to eq("-name" => "string", "age" => "integer")
       end
 
       it "accepts a switch <value> assignment" do
@@ -426,6 +428,7 @@ describe Thor::Options do
 
       it "accepts a switch=<value> assignment" do
         expect(parse("--attributes=a", "b", "c")["attributes"]).to eq(%w(a b c))
+        expect(parse("--attributes=-a", "b", "-c")["attributes"]).to eq(%w(-a b))
       end
 
       it "accepts a switch <value> assignment" do
