@@ -173,6 +173,103 @@ describe Thor::Shell::Basic do
       expect($stdout).to receive(:print).with("this_is_not_a_string\n")
       shell.say(:this_is_not_a_string, nil, true)
     end
+
+    it "does not print a message if muted" do
+      expect($stdout).not_to receive(:print)
+      shell.mute do
+        shell.say("Running...")
+      end
+    end
+
+    it "does not print a message if base is set to quiet" do
+      shell.base = MyCounter.new [1, 2]
+      expect(shell.base).to receive(:options).and_return(:quiet => true)
+
+      expect($stdout).not_to receive(:print)
+      shell.say("Running...")
+    end
+  end
+
+  describe "#say_error" do
+    it "prints a message to the user" do
+      expect($stderr).to receive(:print).with("Running...\n")
+      shell.say_error("Running...")
+    end
+
+    it "prints a message to the user without new line if it ends with a whitespace" do
+      expect($stderr).to receive(:print).with("Running... ")
+      shell.say_error("Running... ")
+    end
+
+    it "does not use a new line with whitespace+newline embedded" do
+      expect($stderr).to receive(:print).with("It's \nRunning...\n")
+      shell.say_error("It's \nRunning...")
+    end
+
+    it "prints a message to the user without new line" do
+      expect($stderr).to receive(:print).with("Running...")
+      shell.say_error("Running...", nil, false)
+    end
+
+    it "coerces everything to a string before printing" do
+      expect($stderr).to receive(:print).with("this_is_not_a_string\n")
+      shell.say_error(:this_is_not_a_string, nil, true)
+    end
+
+    it "does not print a message if muted" do
+      expect($stderr).not_to receive(:print)
+      shell.mute do
+        shell.say_error("Running...")
+      end
+    end
+
+    it "does not print a message if base is set to quiet" do
+      shell.base = MyCounter.new [1, 2]
+      expect(shell.base).to receive(:options).and_return(:quiet => true)
+
+      expect($stderr).not_to receive(:print)
+      shell.say_error("Running...")
+    end
+  end
+
+  describe "#print_wrapped" do
+    let(:message) do
+      "Creates a back-up of the given folder by compressing it in a .tar.gz\n"\
+      "file and then uploading it to the configured Amazon S3 Bucket.\n\n"\
+      "It does not verify the integrity of the generated back-up."
+    end
+
+    before do
+      allow(ENV).to receive(:[]).with("THOR_COLUMNS").and_return(80)
+    end
+
+    context "without indentation" do
+      subject(:wrap_text) { described_class.new.print_wrapped(message) }
+
+      let(:expected_output) do
+        "Creates a back-up of the given folder by compressing it in a .tar.gz file and\n"\
+        "then uploading it to the configured Amazon S3 Bucket.\n\n"\
+        "It does not verify the integrity of the generated back-up.\n"
+      end
+
+      it "properly wraps the text around the 80th column" do
+        expect { wrap_text }.to output(expected_output).to_stdout
+      end
+    end
+
+    context "with indentation" do
+      subject(:wrap_text) { described_class.new.print_wrapped(message, :indent => 4) }
+
+      let(:expected_output) do
+        "    Creates a back-up of the given folder by compressing it in a .tar.gz file\n"\
+        "    and then uploading it to the configured Amazon S3 Bucket.\n\n"\
+        "    It does not verify the integrity of the generated back-up.\n"
+      end
+
+      it "properly wraps the text around the 80th column" do
+        expect { wrap_text }.to output(expected_output).to_stdout
+      end
+    end
   end
 
   describe "#say_status" do
@@ -186,6 +283,20 @@ describe Thor::Shell::Basic do
       shell.say_status(:create, "")
     end
 
+    it "indents a multiline message" do
+      status = :foobar
+      lines = ["first line", "second line", "  third line", "    fourth line"]
+
+      expect($stdout).to receive(:print) do |string|
+        formatted_status = string[/^\s*#{status}\s*/]
+        margin = " " * formatted_status.length
+
+        expect(string).to eq(formatted_status + lines.join("\n#{margin}") + "\n")
+      end
+
+      shell.say_status(status, lines.join("\n") + "\n")
+    end
+
     it "does not print a message if base is muted" do
       expect(shell).to receive(:mute?).and_return(true)
       expect($stdout).not_to receive(:print)
@@ -281,7 +392,7 @@ TABLE
       2.times { @table.first.pop }
       content = capture(:stdout) { shell.print_table(@table) }
       expect(content).to eq(<<-TABLE)
-abc  
+abc#{"  "}
      #0    empty
 xyz  #786  last three
 TABLE
