@@ -173,6 +173,10 @@ describe Thor do
         def exec(*args)
           [options, args]
         end
+
+        def self.exit_on_failure?
+          false
+        end
       end
 
       it "passes remaining args to command when it encounters a non-option" do
@@ -223,6 +227,10 @@ describe Thor do
       def checked(*args)
         [options, args]
       end
+
+      def self.exit_on_failure?
+        false
+      end
     end
 
     it "still accept options and arguments" do
@@ -285,6 +293,10 @@ describe Thor do
       def boring(*args)
         [options, args]
       end
+
+      def self.exit_on_failure?
+        false
+      end
     end
 
     it "does not check the required option in the given command" do
@@ -440,6 +452,9 @@ Usage: "thor scripts:arities:one_arg ARG"'
 Usage: "thor scripts:arities:two_args ARG1 ARG2"'
       arity_asserter.call %w(optional_arg one two), 'ERROR: "thor optional_arg" was called with arguments ["one", "two"]
 Usage: "thor scripts:arities:optional_arg [ARG]"'
+      arity_asserter.call %w(multiple_usages), 'ERROR: "thor multiple_usages" was called with no arguments
+Usage: "thor scripts:arities:multiple_usages ARG --foo"
+       "thor scripts:arities:multiple_usages ARG --bar"'
     end
 
     it "raises an error if the invoked command does not exist" do
@@ -552,6 +567,19 @@ do some fooing
 END
       end
 
+      it "provides full help info when talking about a specific command with multiple usages" do
+        expect(capture(:stdout) { MyScript.command_help(shell, "baz") }).to eq(<<-END)
+Usage:
+  thor my_script:baz THING
+  thor my_script:baz --all
+
+Options:
+  [--all=ALL]  # Do bazing for all the things
+
+super cool
+END
+      end
+
       it "raises an error if the command can't be found" do
         expect do
           MyScript.command_help(shell, "unknown")
@@ -694,12 +722,30 @@ HELP
       expect(klass.start(%w(unknown foo --bar baz))).to eq(%w(foo))
     end
 
-    it "does not check the default type when check_default_type! is not called" do
+    it "issues a deprecation warning on incompatible types by default" do
       expect do
         Class.new(Thor) do
           option "bar", :type => :numeric, :default => "foo"
         end
-      end.not_to raise_error
+      end.to output(/^Deprecation warning/).to_stderr
+    end
+
+    it "allows incompatible types if allow_incompatible_default_type! is called" do
+      expect do
+        Class.new(Thor) do
+          allow_incompatible_default_type!
+
+          option "bar", :type => :numeric, :default => "foo"
+        end
+      end.not_to output.to_stderr
+    end
+
+    it "allows incompatible types if `check_default_type: false` is given" do
+      expect do
+        Class.new(Thor) do
+          option "bar", :type => :numeric, :default => "foo", :check_default_type => false
+        end
+      end.not_to output.to_stderr
     end
 
     it "checks the default type when check_default_type! is called" do
@@ -716,4 +762,19 @@ HELP
       expect(MyScript.start(%w(send))).to eq(true)
     end
   end
+
+  context "without an exit_on_failure? method" do
+    my_script = Class.new(Thor) do
+      desc "no arg", "do nothing"
+      def no_arg
+      end
+    end
+
+    it "outputs a deprecation warning on error" do
+      expect do
+        my_script.start(%w[no_arg one])
+      end.to output(/^Deprecation.*exit_on_failure/).to_stderr
+    end
+  end
+
 end
